package model.geography;

/**
 * GeoLocation is a simple class that models
 * a geographic location on the earth. It contains
 * a latitude (north-south position) and
 * longitude (east-west position)
 * https://en.wikipedia.org/wiki/Geographic_coordinate_system
 *
 * Example Usage:
 *
 * 	//For Brooklyn Center, MN
 * 	GeoLocation geoLocation = GeoLocation.fromLatLong(45.076076, -93.332728);
 * 	prn(geoLocation.toString());
 * */
public class GeoLocation {

	private double latitude;	//north-south position
	private double longitude;	//east-west position
	
	//------------------------------------------
	// Constructing - public	
	
	/** Returns new GeoLocation object with passed latitude and longitude values */
	public static GeoLocation fromLatLong(double lat, double lon) {
		return new GeoLocation(lat, lon);
	}	
	
	/** Returns instance of a GeoLocation with values (0, 0) */
	public static GeoLocation newInstance() {
		return fromLatLong(0, 0);
	}	

	//------------------------------------------
	// Constructing - private

	/** Constructs GeoLocation with passed lat and long values
	 * e.g., new GeoLocation(45.076076, -93.332728) */
	private GeoLocation(double aLatitude, double aLongitude) {
		this.latitude = aLatitude;
		this.longitude = aLongitude;
	}
	
	//------------------------------------------
	// Accessing	

	/** Returns latitude */
	public double getLatitude() {
		return latitude;
	}
	
	/** Returns longitude */	
	public double getLongitude() {
		return longitude;
	}	

	/** Sets latitude to passed parameter */	
	public void setLatitude(double lat) {
		this.latitude = lat;
	}

	/** Sets longitude to passed parameter */	
	public void setLongitude(double lon) {
		this.longitude = lon;
	}

	//------------------------------------------
	// Calculating
	
	/** Returns true if this location is in the northern hemisphere */
	public boolean isInNorthernHemisphere() {
		return this.getLatitude() > 0;
	}
	
	/** Returns false if this location is in the northern hemisphere */
	public boolean isInSouthernHemisphere() {
		return !this.isInNorthernHemisphere();
	}	

	/** Returns perimeter of this rectangle */
	public boolean isOnEquatorHemisphere() {
		return this.getLatitude() == 0;
	}
	
	/** Returns true if this location is north of passed latitude */
	public boolean isNorthOf(double aLatitude) {
		return this.getLatitude() > aLatitude;
	}	
	
	/** Returns true if this location is south of passed latitude */
	public boolean isSouthOf(double aLatitude) {
		return this.getLatitude() < aLatitude;
	}	

	//------------------------------------------
	//Common

	/** Returns "nice string" of this rectangle */
	public String toString() {
		return String.format("(%f, %f)", 
				this.getLatitude(),
				this.getLongitude());
	}

}